@@ -17,7 +17,9 @@ rgasp <- function(design, response,trend=matrix(1,length(response),1),zero.mean=
                   nugget.est=F,range.par=NA,prior_choice='ref_approx',a=0.2,
                   b=1/(length(response))^{1/dim(as.matrix(design))[2]}*(a+dim(as.matrix(design))[2]),
                   kernel_type='matern_5_2',
-                  alpha=rep(1.9,dim(as.matrix(design))[2]),multiple_starts=T,lower_bound=T,max_eval=max(30,20+5*dim(design)[2])){
+                  alpha=rep(1.9,dim(as.matrix(design))[2]),
+                  lower_bound=T,max_eval=max(30,20+5*dim(design)[2]),
+                  initial_values=NA,num_initial_values=2){
   if (zero.mean=="Yes"){
     trend=matrix(0,length(response),1)
   }
@@ -46,11 +48,10 @@ rgasp <- function(design, response,trend=matrix(1,length(response),1),zero.mean=
   model <- new("rgasp")
   model@call <- match.call()
   
-  model@kernel_type <-kernel_type
+  
   #cat('Kernel type is ',model@kernel_type,'\n')
   
-  cat('Multiple starting points: ',multiple_starts, '\n')
-  
+
   model@alpha <-alpha
   
   #####Only numeric inputs are allowed
@@ -69,6 +70,30 @@ rgasp <- function(design, response,trend=matrix(1,length(response),1),zero.mean=
   }
   #  stop("The dimension of the training points and simulator values are different. \n")
   
+
+  ##kernel type
+  if(length(kernel_type)==1){
+    kernel_type=rep(kernel_type,model@p)
+  }else if(length(kernel_type)!=model@p){
+    stop("Please specify the correct number of kernels. \n")
+  }
+  model@kernel_type <-kernel_type
+  
+  ##change kernel type to integer to pass to C++ code
+  ##1 is power exponent, 2 is matern with roughness 3/2, and 3 is matern with roughenss parameter 5/2
+  kernel_type_num=rep(0,  model@p)
+  for(i_p in 1:  model@p){
+    if(model@kernel_type[i_p]=="matern_5_2"){
+       kernel_type_num[i_p]=as.integer(3)
+    }else if (model@kernel_type[i_p]=="matern_3_2"){
+      kernel_type_num[i_p]=as.integer(2)
+    }else if (model@kernel_type[i_p]=="pow_exp"){
+      kernel_type_num[i_p]=as.integer(1)
+    }
+  }
+    
+
+
   #####I now build the gasp emulator here
   ##default setting constant mean basis. Of course We should let user to specify it
   #model@X = matrix(1,model@num_obs,1)   ####constant mean
@@ -102,10 +127,12 @@ rgasp <- function(design, response,trend=matrix(1,length(response),1),zero.mean=
   if(is.na(range.par)){
     ########this also depends on the kernel
     ##get some initial values of beta that we can start from a good set of range parameters to optimize
+    
+    
     COND_NUM_UB = 10^{16}  ###maximum condition number, this might be a little too large
     
     LB_all = optimize(search_LB_prob, interval=c(-5,12), maximum = FALSE, R0=model@R0,COND_NUM_UB= COND_NUM_UB,
-                      p=model@p,kernel_type=model@kernel_type,alpha=model@alpha,nugget=nugget) ###find a lower bound for parameter beta
+                      p=model@p,kernel_type=kernel_type_num,alpha=model@alpha,nugget=nugget) ###find a lower bound for parameter beta
     
     LB_prob = exp(LB_all$minimum)/(exp(LB_all$minimum)+1)
     
@@ -115,10 +142,6 @@ rgasp <- function(design, response,trend=matrix(1,length(response),1),zero.mean=
       LB = c(LB, log(-log(LB_prob)/(max(model@R0[[i_LB]]))))    ###LB is lower bound for log beta, may consider to have it related to p
     }
     
-    
-    
-    ##LB=LB-log(model@p)   #####
-    
     if(lower_bound==T){
       if(model@nugget.est){
         model@LB=c(LB,-Inf)   
@@ -138,139 +161,81 @@ rgasp <- function(design, response,trend=matrix(1,length(response),1),zero.mean=
     ############################the lower bound might be needed to discuss
     
     #beta_initial=(a+model@p)/(model@p*model@CL*b)/2  ###half the prior value
-    beta_initial=50*exp(LB) #####one start
     
-    model@beta_initial=beta_initial
-    
-    if(model@nugget.est){
-      ini_value=c(log(beta_initial), log(0.0001)); ###with nugget
-    }else{
-      ini_value=log(beta_initial);               ###without the nugget
-    }
-    cat('The intial values of range parameters are', 1/exp(ini_value),'\n')
-    
-    cat('Start of the first optimization:','\n')
-    
-    
-    ####not output the message from nlopt
-    #sink(tempfile()) 
-    
-    if(prior_choice=='ref_approx'){####this one can be with nugget or without the nugget
-      #  if (requireNamespace("lbfgs", quietly = TRUE)) {
-      tt_all <- try(nloptr::lbfgs(ini_value, neg_log_marginal_post_approx_ref, 
-                                  neg_log_marginal_post_approx_ref_deriv,nugget=nugget, nugget.est=model@nugget.est, 
-                                  R0=model@R0,X=model@X, zero_mean=model@zero_mean,output=model@output, CL=model@CL, a=a,b=b,
-                                  kernel_type=model@kernel_type,alpha=model@alpha,lower=model@LB,
-                                  nl.info = FALSE, control = list(maxeval=max_eval)),TRUE)
-      #   }
-    }else if(prior_choice=='ref_xi'|prior_choice=='ref_gamma'){####this needs to be revised
-      #  if (requireNamespace("lbfgs", quietly = TRUE)) {
-      tt_all <- try(nloptr::lbfgs(ini_value, neg_log_marginal_post_ref, 
-                                  nugget=nugget, nugget.est=nugget.est, R0=model@R0,
-                                  X=model@X, zero_mean=model@zero_mean,output=model@output, prior_choice=prior_choice, kernel_type=model@kernel_type,
-                                  alpha=model@alpha,lower=model@LB,nl.info = FALSE, control = list(maxeval=max_eval)),TRUE)
-      # }
-    }
-    if(class(tt_all)=="try-error"){
-      #sink()
-      stop(tt_all)
-    }
-    #sink()#### close sink
-    
-    if(model@nugget.est==F){
-      nugget_par=nugget
-    }else{
-      nugget_par=exp(tt_all$par)[model@p+1]
-    }
-    
-    if (tt_all$convergence>0){convergence=T}else{convergence=F}
-    cat('The number of interation is ', tt_all$iter,'\n',
-        'The value of the posterior is ', -tt_all$value,'\n',
-        'Optimized range parameters are', 1/exp(tt_all$par)[1:model@p],'\n',
-        'Optimized nugget parameter is', nugget_par,'\n',
-        'Convergence: ', convergence,'\n' )
-    
-    
-    
-    ###########here I do multiple starts 
-    if(multiple_starts){
-      cat('Start of the second optimization:','\n')
-      
-      
-      beta_initial=(a+model@p)/(model@p*model@CL*b)/2  ###half the prior value
-      
-      #  beta_initial=(a+model@p)/(model@p*model@CL*b)/2 #####another start
-      
-      if(nugget.est==T){
-        ini_value=c(log(beta_initial), log(0.0002)); ###with nugget
-        
-      }else{
-        ini_value=log(beta_initial);               ###without the nugget
+    if(is.na(initial_values)[1]==T){
+      beta_initial=matrix(0,num_initial_values,model@p)
+      eta_initial=rep(0,num_initial_values)
+      beta_initial[1,]=50*exp(LB) #####one start
+      eta_initial[1]=0.0001
+      if(num_initial_values>1){
+        beta_initial[2,]=(a+model@p)/(model@p*model@CL*b)/2  ###half the prior value
+        eta_initial[2]=0.0002
       }
-      cat('The intial values of range parameters are', 1/exp(ini_value),'\n')
-      
-      #sink(tempfile()) 
+      if(num_initial_values>2){
+        for(i_ini in 3:num_initial_values){
+          set.seed(i_ini)
+          beta_initial[i_ini,]=10^3*runif(model@p)/model@CL
+          eta_initial[i_ini]=10^(-3)*runif(1)
+        }
+      }
+      initial_values=cbind(log(beta_initial),log(eta_initial))
+    }
+    
+    model@log_post=-Inf;
+    for(i_ini in 1:num_initial_values){
+      if(model@nugget.est){
+        ini_value=initial_values[i_ini,]
+      }else{
+        ini_value=initial_values[i_ini,1:model@p]
+        ###without the nugget
+      }
+      cat('The initial values of range parameters are', 1/exp(ini_value[1:model@p]),'\n')
+      cat('Start of the optimization ', i_ini,' : \n')
       
       if(prior_choice=='ref_approx'){####this one can be with nugget or without the nugget
-        tt_all2 <- try(nloptr::lbfgs(ini_value, neg_log_marginal_post_approx_ref, 
-                                     neg_log_marginal_post_approx_ref_deriv,nugget=nugget, nugget.est=nugget.est, R0=model@R0,
-                                     X=model@X, zero_mean=model@zero_mean, output=model@output, CL=model@CL, a=a,b=b,  kernel_type=kernel_type,alpha=alpha,lower=model@LB,
-                                     nl.info = FALSE, control = list(maxeval=max_eval)),TRUE)
-        
+        #  if (requireNamespace("lbfgs", quietly = TRUE)) {
+        tt_all <- try(nloptr::lbfgs(ini_value, neg_log_marginal_post_approx_ref, 
+                                    neg_log_marginal_post_approx_ref_deriv,nugget=nugget, nugget.est=model@nugget.est, 
+                                    R0=model@R0,X=model@X, zero_mean=model@zero_mean,output=model@output, CL=model@CL, a=a,b=b,
+                                    kernel_type=kernel_type_num,alpha=model@alpha,lower=model@LB,
+                                    nl.info = FALSE, control = list(maxeval=max_eval)),TRUE)
+        #   }
       }else if(prior_choice=='ref_xi'|prior_choice=='ref_gamma'){####this needs to be revised
-        tt_all2 <- try(nloptr::lbfgs(ini_value, neg_log_marginal_post_ref, 
-                                     nugget=nugget, nugget.est=nugget.est, R0=model@R0,
-                                     X=model@X, zero_mean=model@zero_mean,output=model@output, prior_choice=prior_choice, kernel_type=kernel_type,alpha=alpha,lower=model@LB,
-                                     nl.info = FALSE, control = list(maxeval=max_eval)),TRUE)
+        #  if (requireNamespace("lbfgs", quietly = TRUE)) {
+        tt_all <- try(nloptr::lbfgs(ini_value, neg_log_marginal_post_ref, 
+                                    nugget=nugget, nugget.est=nugget.est, R0=model@R0,
+                                    X=model@X, zero_mean=model@zero_mean,output=model@output, prior_choice=prior_choice, kernel_type=kernel_type_num,
+                                    alpha=model@alpha,lower=model@LB,nl.info = FALSE, control = list(maxeval=max_eval)),TRUE)
+        # }
+      }
+      if(class(tt_all)!="try-error"){
+        if(model@nugget.est==F){
+          nugget_par=nugget
+        }else{
+          nugget_par=exp(tt_all$par)[model@p+1]
+        }
+        
+        if (tt_all$convergence>0){convergence=T}else{convergence=F}
+        cat('The number of interation is ', tt_all$iter,'\n',
+            'The value of the posterior is ', -tt_all$value,'\n',
+            'Optimized range parameters are', 1/exp(tt_all$par)[1:model@p],'\n',
+            'Optimized nugget parameter is', nugget_par,'\n',
+            'Convergence: ', convergence,'\n' )
         
         
+        if( (-tt_all$value)>model@log_post){
+          log_lik=-tt_all$value
+          model@log_post=-tt_all$value
+          model@nugget=nugget;
+          if(nugget.est){
+            model@beta_hat = exp(tt_all$par)[1:model@p];
+            model@nugget=exp(tt_all$par)[model@p+1];
+          }else{
+            model@beta_hat = exp(tt_all$par);
+            #  model@nugget=0;
+          }
+        }
       }
-      
-      
-      if(class(tt_all2)=="try-error"){
-        #sink()
-        stop(tt_all2)
-      }
-      
-      #sink()#### close sink
-      
-      
-      if(model@nugget.est==F){
-        nugget_par2=nugget
-      }else{
-        nugget_par2=exp(tt_all2$par)[model@p+1]
-      }
-      
-      if (tt_all2$convergence>0){convergence2=T}else{convergence2=F}
-      cat('The number of interation is ', tt_all2$iter,'\n',
-          'The value of the posterior is ', -tt_all2$value,'\n',
-          'Optimized range parameters are', 1/exp(tt_all2$par)[1:model@p],'\n',
-          'Optimized nugget parameter is', nugget_par2,'\n',
-          'Convergence: ', convergence2,'\n' )
-      
-      if(tt_all2$value<tt_all$value){
-        model@beta_initial=beta_initial
-        tt_all=tt_all2
-      }
-    }
-    
-    
-    
-    ################################
-    ##############estimated beta
-    # print out to see whether or not it has converged
-    # cat('Convergence: ',tt_all,'\n')  ###cat doesn't work here
-    
-    
-    model@log_post=-tt_all$value
-    
-    model@nugget=nugget;
-    if(nugget.est){
-      model@beta_hat = exp(tt_all$par)[1:model@p];
-      model@nugget=exp(tt_all$par)[model@p+1];
-    }else{
-      model@beta_hat = exp(tt_all$par);
-      #  model@nugget=0;
     }
   }else{###this is the case where the range parameters and the nugget are all fixed
     model@LB=rep(-Inf,model@p)
@@ -279,7 +244,7 @@ rgasp <- function(design, response,trend=matrix(1,length(response),1),zero.mean=
   }
   
   list_return=construct_rgasp(model@beta_hat, model@nugget, model@R0, model@X, zero_mean=model@zero_mean,
-                              model@output,model@kernel_type,model@alpha); 
+                              model@output,kernel_type_num,model@alpha); 
   model@L=list_return[[1]];
   model@LX=list_return[[2]];
   model@theta_hat=list_return[[3]];
@@ -293,17 +258,11 @@ show.rgasp <- function(object) {
   cat("Call:\n")
   print(object@call)
   
-  # cat('The dimension names of the design are: ',colnames(object@input),'\n')
-  #  cat('The dimension names of the response is: ',colnames(object@output),'\n')
-  
-  #cat('The dimension  of the design is: ',dim(object@input),'\n')
-  #cat('The number of the output is: ',dim(object@output),'\n')
   cat('Mean parameters: ',object@theta_hat,'\n')
   cat('Variance parameter: ', object@sigma2_hat,'\n')
   cat('Range parameters: ', 1/object@beta_hat,'\n')
   cat('Noise parameter: ', object@sigma2_hat*object@nugget,'\n')
   
   
-  # cat('The slots of this object are: ',slotNames(object),'\n')
-  
+
 }
\ No newline at end of file
